/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef nsTransactionManagerCID_h__
#define nsTransactionManagerCID_h__

#define NS_TRANSACTIONMANAGER_CID                   \
{ /* 9C8F9601-801A-11d2-98BA-00805F297D89 */        \
0x9c8f9601, 0x801a, 0x11d2,                         \
{ 0x98, 0xba, 0x0, 0x80, 0x5f, 0x29, 0x7d, 0x89 } }

#endif /* nsTransactionManagerCID_h__ */
