/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */


#include "nsISupports.idl"

interface nsIURI;
interface nsIFile;
interface nsIInterfaceRequestor;
interface nsIArray;

[scriptable, uuid(e7bcc22c-e9fc-4e7d-88b9-7482399b322d)]
interface nsIParentalControlsService : nsISupports
{
  /**
   * Action types that can be blocked for users.
   */
  const short DOWNLOAD = 1; // Downloading files
  const short INSTALL_EXTENSION = 2; // Installing extensions
  const short INSTALL_APP = 3; // Installing webapps
  const short VISIT_FILE_URLS = 4; // Opening file:/// urls
  const short SHARE = 5; // Sharing
  const short BOOKMARK = 6; // Creating bookmarks
  const short ADD_CONTACT = 7; // Add contacts to the system database
  const short SET_IMAGE = 8; // Setting images as wall paper
  const short MODIFY_ACCOUNTS = 9; // Modifying system accounts
  const short REMOTE_DEBUGGING = 10; // Remote debugging
  const short IMPORT_SETTINGS = 11; // Importing settings from other apps

  /**
   * @returns true if the current user account has parental controls
   * restrictions enabled.
   */ 
  readonly attribute boolean parentalControlsEnabled;

  /**
   * @returns true if the current user account parental controls
   * restrictions include the blocking of all file downloads.
   */ 
  readonly attribute boolean blockFileDownloadsEnabled;

  /**
   * Check if the user can do the prescibed action for this uri.
   *
   * @param aAction             Action being performed
   * @param aUri                The uri requesting this action
   * @param aWindow             The window generating this event.
   */
   boolean isAllowed(in short aAction, [optional] in nsIURI aUri);

  /**
   * Request that blocked URI(s) be allowed through parental
   * control filters. Returns true if the URI was successfully
   * overriden. Note, may block while native UI is shown.
   *
   * @param aTarget(s)          URI to be overridden. In the case of
   *                            multiple URI, the first URI in the array
   *                            should be the root URI of the site.
   * @param window              Window that generates the event.
   */
  boolean requestURIOverride(in nsIURI aTarget, [optional] in nsIInterfaceRequestor aWindowContext);
  boolean requestURIOverrides(in nsIArray aTargets, [optional] in nsIInterfaceRequestor aWindowContext);

  /**
   * @returns true if the current user account has parental controls
   * logging enabled. If true, applications should log relevent events
   * using 'log'.
   */ 
  readonly attribute boolean loggingEnabled;

  /**
   * Log entry types. Additional types can be defined and implemented
   * as needed. Other possible event types might include email events,
   * media related events, and IM events. 
   */
  const short ePCLog_URIVisit = 1;    /* Web content */
  const short ePCLog_FileDownload = 2;  /* File downloads */

  /**
   * Log an application specific parental controls
   * event.
   *
   * @param aEntryType       Constant defining the type of event.
   * @param aFlag            A flag indicating if the subject content
   *                         was blocked.
   * @param aSource          The URI source of the subject content.
   * @param aTarget          The location the content was saved to if
   *                         no blocking occurred.
   */
  void log(in short aEntryType, in boolean aFlag, in nsIURI aSource, [optional] in nsIFile aTarget);
};
